Class: Rhales::RueDocument

Inherits:
Object
  • Object
show all
Defined in:
lib/rhales/rue_document.rb

Overview

High-level interface for parsed .rue files

This class provides a convenient interface to .rue files parsed by RueFormatParser. It uses RueFormatParser internally for low-level parsing and provides high-level methods for accessing sections, attributes, and extracted data.

Features:

  • High-level interface to RueFormatParser AST

  • Accurate error reporting with line/column information

  • Convenient section access methods

  • Section validation and attribute extraction

  • Variable and partial dependency analysis

  • AST-to-string conversion when needed

Note: This class represents a parsed .rue file document, similar to how HTML::Document represents a parsed HTML document.

Usage:

document = RueDocument.new(rue_content)
document.parse!
template_section = document.section('template')
variables = document.template_variables

Defined Under Namespace

Classes: InvalidSyntaxError, ParseError, SectionDuplicateError, SectionMissingError

Constant Summary collapse

REQUIRES_ONE_OF_SECTIONS =

At least one of these sections must be present

%w[data template].freeze
KNOWN_SECTIONS =
%w[data template logic].freeze
ALL_SECTIONS =
KNOWN_SECTIONS.freeze
KNOWN_DATA_ATTRIBUTES =

Known data section attributes

%w[window merge layout].freeze

Instance Attribute Summary collapse

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(content, file_path = nil) ⇒ RueDocument

Returns a new instance of RueDocument.



44
45
46
47
48
49
# File 'lib/rhales/rue_document.rb', line 44

def initialize(content, file_path = nil)
  @content   = content
  @file_path = file_path
  @grammar   = RueFormatParser.new(content, file_path)
  @ast       = nil
end

Instance Attribute Details

#astObject (readonly)

Returns the value of attribute ast.



42
43
44
# File 'lib/rhales/rue_document.rb', line 42

def ast
  @ast
end

#contentObject (readonly)

Returns the value of attribute content.



42
43
44
# File 'lib/rhales/rue_document.rb', line 42

def content
  @content
end

#file_pathObject (readonly)

Returns the value of attribute file_path.



42
43
44
# File 'lib/rhales/rue_document.rb', line 42

def file_path
  @file_path
end

#grammarObject (readonly)

Returns the value of attribute grammar.



42
43
44
# File 'lib/rhales/rue_document.rb', line 42

def grammar
  @grammar
end

Class Method Details

.parse_file(file_path) ⇒ Object

Raises:

  • (ArgumentError)


292
293
294
295
296
297
# File 'lib/rhales/rue_document.rb', line 292

def parse_file(file_path)
  raise ArgumentError, 'Not a .rue file' unless rue_file?(file_path)

  file_content = File.read(file_path)
  new(file_content, file_path).parse!
end

.rue_file?(file_path) ⇒ Boolean

Returns:

  • (Boolean)


299
300
301
# File 'lib/rhales/rue_document.rb', line 299

def rue_file?(file_path)
  File.extname(file_path) == '.rue'
end

Instance Method Details

#all_variablesObject



160
161
162
# File 'lib/rhales/rue_document.rb', line 160

def all_variables
  (template_variables + data_variables).uniq
end

#convert_node_to_string(node) ⇒ Object



72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
# File 'lib/rhales/rue_document.rb', line 72

def convert_node_to_string(node)
  case node.type
  when :text
    node.value
  when :variable_expression
    name = node.value[:name]
    raw  = node.value[:raw]
    raw ? "{{{#{name}}}}" : "{{#{name}}}"
  when :partial_expression
    "{{> #{node.value[:name]}}}"
  when :if_block
    condition    = node.value[:condition]
    if_content   = convert_nodes_to_string(node.value[:if_content])
    else_content = convert_nodes_to_string(node.value[:else_content])
    if else_content.empty?
      "{{#if #{condition}}}#{if_content}{{/if}}"
    else
      "{{#if #{condition}}}#{if_content}{{else}}#{else_content}{{/if}}"
    end
  when :unless_block
    condition = node.value[:condition]
    content   = convert_nodes_to_string(node.value[:content])
    "{{#unless #{condition}}}#{content}{{/unless}}"
  when :each_block
    items   = node.value[:items]
    content = convert_nodes_to_string(node.value[:content])
    "{{#each #{items}}}#{content}{{/each}}"
  when :handlebars_expression
    # Handle legacy format for data sections
    if node.value[:raw]
      "{{{#{node.value[:content]}}}"
    else
      "{{#{node.value[:content]}}}"
    end
  else
    ''
  end
end

#convert_nodes_to_string(nodes) ⇒ Object



68
69
70
# File 'lib/rhales/rue_document.rb', line 68

def convert_nodes_to_string(nodes)
  nodes.map { |node| convert_node_to_string(node) }.join
end

#data_attributesObject



115
116
117
# File 'lib/rhales/rue_document.rb', line 115

def data_attributes
  @data_attributes ||= {}
end

#data_variablesObject



156
157
158
# File 'lib/rhales/rue_document.rb', line 156

def data_variables
  extract_variables_from_section('data')
end

#layoutObject



131
132
133
# File 'lib/rhales/rue_document.rb', line 131

def layout
  data_attributes['layout']
end

#merge_strategyObject



127
128
129
# File 'lib/rhales/rue_document.rb', line 127

def merge_strategy
  data_attributes['merge']
end

#parse!Object



51
52
53
54
55
56
57
58
# File 'lib/rhales/rue_document.rb', line 51

def parse!
  @grammar.parse!
  @ast = @grammar.ast
  parse_data_attributes!
  self
rescue RueFormatParser::ParseError => ex
  raise ParseError, "Parser error: #{ex.message}"
end

#partialsObject



144
145
146
147
148
149
150
# File 'lib/rhales/rue_document.rb', line 144

def partials
  return [] unless @ast

  partials = []
  extract_partials_from_node(@ast, partials)
  partials.uniq
end

#schema_pathObject



123
124
125
# File 'lib/rhales/rue_document.rb', line 123

def schema_path
  data_attributes['schema']
end

#section(name) ⇒ Object



111
112
113
# File 'lib/rhales/rue_document.rb', line 111

def section(name)
  sections[name]
end

#section?(name) ⇒ Boolean

Returns:

  • (Boolean)


135
136
137
# File 'lib/rhales/rue_document.rb', line 135

def section?(name)
  @grammar.sections.key?(name)
end

#section_node(name) ⇒ Object

Get the raw section node with location information



140
141
142
# File 'lib/rhales/rue_document.rb', line 140

def section_node(name)
  @grammar.sections[name]
end

#sectionsObject



60
61
62
63
64
65
66
# File 'lib/rhales/rue_document.rb', line 60

def sections
  return {} unless @ast

  @grammar.sections.transform_values do |section_node|
    convert_nodes_to_string(section_node.value[:content])
  end
end

#template_variablesObject



152
153
154
# File 'lib/rhales/rue_document.rb', line 152

def template_variables
  extract_variables_from_section('template', exclude_partials: true)
end

#window_attributeObject



119
120
121
# File 'lib/rhales/rue_document.rb', line 119

def window_attribute
  data_attributes['window'] || 'data'
end